import numpy as np
import ctypes
from cryoCOFI import get_lib_path

def hough_transform_for_radius(edge_image, radius):

    edge_image = edge_image.astype(np.uint8)
    rows, cols = edge_image.shape

    # lib = ctypes.CDLL('./lib/weighted_hough_transform.so')
    lib = ctypes.CDLL(get_lib_path('weighted_hough_transform'))

    lib.hough_transform_for_radius.argtypes = [
        ctypes.POINTER(ctypes.c_ubyte),
        ctypes.c_int,
        ctypes.c_int,
        ctypes.c_int,
        ctypes.POINTER(ctypes.c_int),
        ctypes.POINTER(ctypes.c_int),
        ctypes.POINTER(ctypes.c_int),
    ]
    lib.hough_transform_for_radius.restype = None

    edge_image_ptr = edge_image.ctypes.data_as(ctypes.POINTER(ctypes.c_ubyte))

    best_a = ctypes.c_int()
    best_b = ctypes.c_int()

    hough_max_x = 2 * radius + cols
    hough_max_y = 2 * radius + rows
    accumulator_size = hough_max_x * hough_max_y

    accumulator = np.zeros((hough_max_y, hough_max_x), dtype=np.int32)
    accumulator_ptr = accumulator.ctypes.data_as(ctypes.POINTER(ctypes.c_int))

    lib.hough_transform_for_radius(
        edge_image_ptr,
        ctypes.c_int(rows),
        ctypes.c_int(cols),
        ctypes.c_int(radius),
        ctypes.byref(best_a),
        ctypes.byref(best_b),
        accumulator_ptr
    )

    return best_a.value, best_b.value, accumulator



from .carbon_film_detector import *
from .detector_for_dynamo import multi_mrc_processing_dynamo
import argparse

def detector_for_relion():
    pass
    
def main():
    parser = argparse.ArgumentParser(description='''
    -----------------------------------
    cryoCOFI: CarbOn FIlm detector for cryo-EM images
    -----------------------------------
    ''',
    formatter_class=argparse.RawTextHelpFormatter,
    epilog='''
    -----------------------------------
    Email: zhen.victor.huang@gmail.com if you have any questions. 
    Please visit https://github.com/ZhenHuangLab/cryoCOFI for more information.
    -----------------------------------
    ''')
    subparsers = parser.add_subparsers(dest='command', help='Please specify the command to run!')

    # readmrc
    readmrc_parser = subparsers.add_parser('readmrc', help='Read MRC file and detect carbon film')
    readmrc_parser.add_argument('--input', '-i', type=str, required=True, help='Input MRC file')  
    readmrc_parser.add_argument('--lowpass', '-lp', type=int, default=200, help='Low pass filter cutoff angstrom')
    readmrc_parser.add_argument('--kernel_radius', '-kr', type=int, default=5, help='Kernel radius for bilateral filter')
    readmrc_parser.add_argument('--sigma_color', '-sc', type=float, default=10.0, help='Sigma color for bilateral filter')
    readmrc_parser.add_argument('--sigma_space', '-ss', type=float, default=10.0, help='Sigma space for bilateral filter')
    readmrc_parser.add_argument('--diameter', '-d', type=int, default=12000, help='Carbon Hole Diameter in Angstrom')
    readmrc_parser.add_argument('--edge', '-e', type=int, default=20, help='Removing edge pixels and cropping the image')
    readmrc_parser.add_argument('--mode_threshold', '-mt', type=float, default=0, help='Mode threshold for finding the carbon film edge')
    readmrc_parser.add_argument('--edge_quotient_threshold', '-eqt', type=float, default=6, help='Edge quotient threshold for finding the carbon film edge')
    readmrc_parser.add_argument('--show_fig', '-sf', action='store_true', default=False, help='Show figures if specified')
    readmrc_parser.add_argument('--verbose', '-v', action='store_true', default=False, help='Show verbose information if specified')
    
    readdynamo_parser = subparsers.add_parser('readdynamo', help='Read Dynamo doc and tbl file and output a new tbl file without particles inside the carbon film')
    readdynamo_parser.add_argument('--doc_path', '-doc', type=str, required=True, help='Input Dynamo .doc file')
    readdynamo_parser.add_argument('--tbl_path', '-tbl', type=str, required=True, help='Input Dynamo .tbl file')
    readdynamo_parser.add_argument('--out_path', '-out', type=str, required=True, help='Output Dynamo .tbl file')
    readdynamo_parser.add_argument('--low_pass', '-lp', type=int, default=200, help='Low pass filter cutoff angstrom')
    readdynamo_parser.add_argument('--kernel_radius', '-kr', type=int, default=5, help='Kernel radius for bilateral filter')
    readdynamo_parser.add_argument('--sigma_color', '-sc', type=float, default=10.0, help='Sigma color for bilateral filter')
    readdynamo_parser.add_argument('--sigma_space', '-ss', type=float, default=10.0, help='Sigma space for bilateral filter')
    readdynamo_parser.add_argument('--diameter', '-d', type=int, default=12000, help='Carbon Hole Diameter in Angstrom')
    readdynamo_parser.add_argument('--edge', '-e', type=int, default=20, help='Removing edge pixels and cropping the image')
    readdynamo_parser.add_argument('--mode_threshold', '-mt', type=float, default=0, help='Mode threshold for finding the carbon film edge')
    readdynamo_parser.add_argument('--edge_quotient_threshold', '-eqt', type=float, default=6, help='Edge quotient threshold for finding the carbon film edge')
    readdynamo_parser.add_argument('--verbose', '-v', action='store_true', default=False, help='Show verbose information if specified')

    args = parser.parse_args()

    if args.command == 'readmrc':
        detector_for_mrc(
            args.input,
            args.lowpass,
            args.kernel_radius,
            args.sigma_color,
            args.sigma_space,
            args.diameter,
            args.edge,
            args.mode_threshold,
            args.edge_quotient_threshold,
            args.show_fig,
            args.verbose
        )
    elif args.command == 'readrelion':
        detector_for_relion()
    elif args.command == 'readdynamo':
        multi_mrc_processing_dynamo(
            args.doc_path,
            args.tbl_path,
            args.out_path,
            args.low_pass,
            args.kernel_radius,
            args.sigma_color,
            args.sigma_space,
            args.diameter,
            args.edge,
            args.mode_threshold,
            args.edge_quotient_threshold,
            args.verbose
        )
    else:
        parser.print_help()

if __name__ == '__main__':
    main()


import cupy as cp
import numpy as np
import mrcfile
import argparse
import matplotlib.pyplot as plt
from .average_z import average_along_z
from cryoCOFI import get_lib_path
from .low_pass_filter import low_pass_filter_gaussian
import ctypes

def read_pixel_size(mrc_path):
    with mrcfile.open(mrc_path) as mrc:
        pixel_size = mrc.voxel_size.x
    return pixel_size

def bilateral_filter(input_image, kernel_radius, sigma_color, sigma_space):
    height, width = input_image.shape
    input_image = input_image.astype(np.float32)

    output_image = np.zeros_like(input_image)

    # lib = ctypes.CDLL('./lib/bilateral_filter.so')
    lib = ctypes.CDLL(get_lib_path('bilateral_filter'))
    lib.bilateral_filter.argtypes = [
        ctypes.POINTER(ctypes.c_float),  
        ctypes.POINTER(ctypes.c_float), 
        ctypes.c_int, 
        ctypes.c_int, 
        ctypes.c_int, 
        ctypes.c_float,  
        ctypes.c_float  
    ]
    lib.bilateral_filter.restype = None 

    input_ptr = input_image.ctypes.data_as(ctypes.POINTER(ctypes.c_float))
    output_ptr = output_image.ctypes.data_as(ctypes.POINTER(ctypes.c_float))

    lib.bilateral_filter(
        input_ptr,
        output_ptr,
        ctypes.c_int(width),
        ctypes.c_int(height),
        ctypes.c_int(kernel_radius),
        ctypes.c_float(sigma_color),
        ctypes.c_float(sigma_space)
    )

    return output_image

def gaussian_filter(input_image, kernel_size):

    input_image = input_image.astype(np.float32)
    height, width = input_image.shape

    output_image = np.zeros_like(input_image)

    # lib = ctypes.CDLL('./lib/gaussian_filter.so')
    lib = ctypes.CDLL(get_lib_path('gaussian_filter'))

    lib.gaussian_filter.argtypes = [
        ctypes.POINTER(ctypes.c_float),  
        ctypes.POINTER(ctypes.c_float), 
        ctypes.c_int, 
        ctypes.c_int, 
        ctypes.c_int
    ]
    lib.gaussian_filter.restype = None  

    input_ptr = input_image.ctypes.data_as(ctypes.POINTER(ctypes.c_float))
    output_ptr = output_image.ctypes.data_as(ctypes.POINTER(ctypes.c_float))

    lib.gaussian_filter(
        input_ptr,
        output_ptr,
        ctypes.c_int(width),
        ctypes.c_int(height),
        ctypes.c_int(kernel_size)
    )

    return output_image

def edge_detector(image):
    image = image.astype(np.float32)
    height, width = image.shape

    edge_map = np.zeros((height, width), dtype=np.uint8)

    # lib = ctypes.CDLL('./lib/edge_detector.so')
    lib = ctypes.CDLL(get_lib_path('edge_detector'))

    lib.edge_detector.argtypes = [
        ctypes.POINTER(ctypes.c_float),
        ctypes.POINTER(ctypes.c_ubyte),
        ctypes.c_int,
        ctypes.c_int
    ]
    lib.edge_detector.restype = None

    input_ptr = image.ctypes.data_as(ctypes.POINTER(ctypes.c_float))
    output_ptr = edge_map.ctypes.data_as(ctypes.POINTER(ctypes.c_ubyte))

    lib.edge_detector(input_ptr, output_ptr, ctypes.c_int(width), ctypes.c_int(height))

    return edge_map


if __name__ == "__main__":
    
    parser = argparse.ArgumentParser()

    parser.add_argument('--input', '-i', type=str)
    parser.add_argument('--lowpass', '-lp', type=int, default=200)

    args = parser.parse_args()
    tg_path = args.input
    low_pass = args.lowpass

    with mrcfile.open(tg_path) as mrc:
        data = mrc.data
    pixel_size = read_pixel_size(tg_path)
    data = cp.array(data)
    average = average_along_z(tg_path)

    lp_filtered_img = low_pass_filter_gaussian(average, low_pass, pixel_size)

    lp_bi_img = bilateral_filter(lp_filtered_img.get(), kernel_radius=5, sigma_color=10.0, sigma_space=10.0)

    lp_g_img = gaussian_filter(lp_filtered_img.get(), kernel_size=5)

    plt.figure()
    plt.subplot(1, 3, 1)
    plt.imshow(average.get(), cmap='gray')
    plt.title('Original Image')
    plt.axis('off')
    plt.subplot(1, 3, 2)
    # plt.imshow(lp_filtered_img.get(), cmap='gray')
    # plt.title('Low-pass Filtered Image')
    # plt.axis('off')

    # plt.subplot(1, 3, 3)
    plt.imshow(lp_bi_img, cmap='gray')
    plt.title('Bilateral Filtered Image')
    plt.axis('off')

    # plt.subplot(1, 3, 3)
    # plt.imshow(lp_g_img, cmap='gray')
    # plt.title('Gaussian Filtered Image')
    # plt.axis('off')
    plt.subplot(1, 3, 3)
    plt.imshow(edge_detector(lp_bi_img), cmap='gray')
    plt.title('Edge Image')
    plt.axis('off')
    plt.show()




import cupy as cp
import matplotlib.pyplot as plt
import mrcfile
import argparse

def average_along_z(tg_path, start=0, end=None):
    with mrcfile.open(tg_path) as mrc:
        data = mrc.data
    data = cp.array(data)
    if end is not None:
        data = data[start:end, :, :]
    # Average all the 2D images along z-axis
    average = cp.mean(data, axis=0)
    return average

if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument('--tomo_path', '-t', type=str)
    parser.add_argument('--start', '-s', type=int, default=0)
    parser.add_argument('--end', '-e', type=int, default=None)
    parser.add_argument('--output', '-o', type=str)
    args = parser.parse_args()

    tg_path = args.tomo_path
    start = args.start
    end = args.end
    output = args.output

    average = average_along_z(tg_path, start, end)

    with mrcfile.new(output, overwrite=True) as mrc:
        mrc.set_data(cp.asnumpy(average))

    # average = low_pass_filter(average, 20, pixel_size)
    # average = gaussian_filter(average, sigma=1)
    # Show the average image
    # plt.subplot(121)
    # plt.imshow(average.get(), cmap='gray')
    # plt.axis('off')
    # plt.subplot(122)
    # plt.imshow(average_part.get(), cmap='gray')
    # plt.axis('off')
    # plt.show()


import numpy as np
import ctypes
from cryoCOFI import get_lib_path

def find_highest_density(img, mask):
    if img.ndim != 2 or mask.ndim != 2:
        raise ValueError("Both img and mask must be 2D numpy arrays.")
    
    # Load the CUDA library
    # cuda_lib = ctypes.CDLL(os.path.abspath("./lib/find_highest_density_double.so"))
    cuda_lib = ctypes.CDLL(get_lib_path('find_highest_density_double'))
    # Define the function prototype
    cuda_lib.find_highest_density_cuda.argtypes = [
        np.ctypeslib.ndpointer(dtype=np.float64, flags="C_CONTIGUOUS"),
        np.ctypeslib.ndpointer(dtype=np.int32, flags="C_CONTIGUOUS"),
        ctypes.c_int
    ]
    cuda_lib.find_highest_density_cuda.restype = ctypes.c_double

    # Ensure the arrays are contiguous and have the correct data types
    img_flat = np.ascontiguousarray(img.flatten(), dtype=np.float64)
    mask_flat = np.ascontiguousarray(mask.flatten(), dtype=np.int32)
    
    # Call the CUDA function
    diff = cuda_lib.find_highest_density_cuda(img_flat, mask_flat, img_flat.size)
    
    return diff

# def find_highest_density(img, mask):
#     cuda_lib = ctypes.CDLL(os.path.abspath("./lib/find_highest_density.so"))

#     # Define the function prototype
#     cuda_lib.find_highest_density_cuda.argtypes = [
#         np.ctypeslib.ndpointer(dtype=np.float32, flags="C_CONTIGUOUS"),
#         np.ctypeslib.ndpointer(dtype=np.int32, flags="C_CONTIGUOUS"),
#         ctypes.c_int
#     ]
#     cuda_lib.find_highest_density_cuda.restype = ctypes.c_float

#     if img.ndim != 2 or mask.ndim != 2:
#         raise ValueError("Both img and mask must be 2D numpy arrays.")
    
#     # Ensure the arrays are contiguous and have the correct data types
#     img_flat = np.ascontiguousarray(img.flatten(), dtype=np.float32)
#     mask_flat = np.ascontiguousarray(mask.flatten(), dtype=np.int32)
    
#     # Call the CUDA function
#     diff = cuda_lib.find_highest_density_cuda(img_flat, mask_flat, img_flat.size)
#     return diff


import cupy as cp
import mrcfile

def read_pixel_size(mrc_path):
    with mrcfile.open(mrc_path) as mrc:
        pixel_size = mrc.voxel_size.x
    return pixel_size

def flag_masking(img, a, b, radius, edge=20, edge_flag=2, edge_map_flag=3):
    # generate a mask for calculating the average density inside and outside the circle
    mask = cp.zeros_like(img)
    y, x = cp.ogrid[:img.shape[0], :img.shape[1]]
    distance_map = cp.sqrt((x - b)**2 + (y - a)**2)
    mask[distance_map < radius - 1 ] = 1

    mask[(distance_map > radius - 1) & (distance_map < radius + 1)] = edge_map_flag

    mask[0:edge, :] = edge_flag
    mask[img.shape[0]-edge:img.shape[0], :] = edge_flag
    mask[:, 0:edge] = edge_flag
    mask[:, img.shape[1]-edge:img.shape[1]] = edge_flag

    return mask



from .carbon_film_detector import detector_for_mrc
import tqdm
import numpy as np
import os
import pandas as pd

def read_dynamo_doc(doc_path):
    # 获取一个dict，key为mrc_path_index，value为mrc_path
    mrc_index_paths = {}
    with open(doc_path, 'r') as file:
        for line in file:
            parts = line.strip().split()
            if len(parts) > 1:
                mrc_path_index = int(parts[0])
                mrc_path = ' '.join(parts[1:])
                mrc_index_paths[mrc_path_index] = mrc_path
    return mrc_index_paths

def read_dynamo_tbl(tbl_path):
    # Define column dtypes
    dtypes = {i: int for i in range(40)}  # Set all columns to int by default
    float_columns = [6, 7, 8, 23, 24, 25]  # Columns to be set as float (0-based index)
    for col in float_columns:
        dtypes[col] = float

    # Read the file with specified dtypes
    df = pd.read_csv(tbl_path, sep=' ', header=None, dtype=dtypes)
    return df

def save_dynamo_tbl(df, out_path):
    # Convert int columns to int (in case they were changed to float during processing)
    int_columns = [col for col in df.columns if col not in [6, 7, 8, 23, 24, 25]]
    df[int_columns] = df[int_columns].astype(int)

    # Save the DataFrame without index and header
    df.to_csv(out_path, sep=' ', header=None, index=None, float_format='%.5f')

def read_dynamo_tbl_tomogram_index(df):
    # 读取.tbl文件，得到表格，返回第19列的值，代表tomogram的index
    # 去除重复的tomogram_index
    tomogram_indices = df[19].unique()
    return tomogram_indices

def read_dynamo_tbl_particle_list(df, tomogram_index):
    # 读取tomogram_indices列表，搜索到tomogram_index相同值的行，返回序号
    # 读取.tbl文件，得到表格，根据上述行序号，返回那一段table slice

    # 找到与tomogram_index相同的行
    mask = df[19] == tomogram_index
    # 根据mask，得到行序号
    row_indices = np.where(mask)[0]
    # 根据行序号，得到table slice
    df_slice = df.iloc[row_indices]
    
    return df_slice

def multi_mrc_processing_dynamo(doc_path,
                        tbl_path,
                        out_path,
                        low_pass,
                        kernel_radius,
                        sigma_color,
                        sigma_space,
                        diameter,
                        edge,
                        mode_threshold,
                        edge_quotient_threshold,
                        verbose):

    mrc_index_paths = read_dynamo_doc(doc_path)

    df = read_dynamo_tbl(tbl_path)

    # Create df_modified with the same dtypes as df
    df_modified = pd.DataFrame(columns=df.columns).astype(df.dtypes)

    tomogram_indices = read_dynamo_tbl_tomogram_index(df)

    for tomogram_index in tqdm.tqdm(tomogram_indices, desc="Processing tomograms", position=0, dynamic_ncols=True, unit="tg"):
        df_slice = read_dynamo_tbl_particle_list(df, tomogram_index)

        mrc_path = mrc_index_paths[tomogram_index]

        if not os.path.exists(mrc_path):
            print(f"Warning: Skip tomogram {tomogram_index} {mrc_path} because not exist.")
            df_modified = pd.concat([df_modified, df_slice], ignore_index=True)
            continue

        mask = detector_for_mrc(mrc_path,
                                low_pass,
                                kernel_radius,
                                sigma_color,
                                sigma_space,
                                diameter,
                                edge,
                                mode_threshold,
                                edge_quotient_threshold,
                                show_fig=False,
                                verbose=verbose)
        if mask is False:
            # 如果没有碳膜，不会动原本的particle，直接将df_slice添加到df_modified
            df_modified = pd.concat([df_modified, df_slice], ignore_index=True)
            if verbose:
                print(f"Skip tomogram {tomogram_index} {mrc_path} because no carbon film detected.")
        else:
            if verbose:
                print(f"Processing tomogram {tomogram_index} {mrc_path} with carbon film detected.")
            # 有碳膜时，需要根据path的序号实现另一个循环，判断tomogram中的xy坐标是在圆弧内还是圆弧外，在圆弧外则从列表中删去该坐标，在圆弧内则保留
            for _, row in tqdm.tqdm(df_slice.iterrows(), desc="Screening particles", position=1, dynamic_ncols=True, unit="ptcl", leave=False):
                x = row[23]
                y = row[24]
                if mask[y, x] == 1:
                    df_modified = pd.concat([df_modified, pd.DataFrame(row).T], ignore_index=True)

    save_dynamo_tbl(df_modified, out_path)
    print(f"New tbl file saved to {out_path}.")


from setuptools import setup, find_packages

setup(
    name='cryoCOFI',
    version='0.1.1',
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    install_requires=[
        'cupy>=13.3.0',
        'mrcfile>=1.5.3',
        'matplotlib>=3.9.2',
        'argparse',
        'numpy>=2.0.2',
        'tqdm>=4.66.5',
        'pandas>=2.2.3'
    ],
    package_data={
        'cryoCOFI': ['lib/*.so'],
    },
    include_package_data=True,
    license='GPLv3',
    description='CarbOn FIlm detector for cryo-EM images',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Zhen Huang',
    author_email='zhen.victor.huang@gmail.com',
    url='https://github.com/ZhenHuangLab/cryoCOFI',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Development Status :: 3 - Alpha',
        'Environment :: GPU :: NVIDIA CUDA :: 12 :: 12.2',
    ],
    python_requires='>=3.9',
    entry_points={
        'console_scripts': [
            'cryoCOFI=cryoCOFI.main:main'
        ],
    },
    exclude_package_data={
        '': ['__pycache__'],
        '**': ['__pycache__', '*.py[co]'],
    },
    keywords='cryo-EM, cryo-ET, carbon film, edge detection, CUDA, CuPy',
    project_urls={
        'Bug Reports': 'https://github.com/ZhenHuangLab/cryoCOFI/issues',
        'Source': 'https://github.com/ZhenHuangLab/cryoCOFI',
    },
)



import cupy as cp
import matplotlib.pyplot as plt
import mrcfile
from .low_pass_filter import low_pass_filter_gaussian
from .hough_transform import hough_transform_for_radius
from .bicanny import *
from .average_z import average_along_z
from ._utils import read_pixel_size, flag_masking
from .find_highest_density import find_highest_density

def detector_for_mrc(tg_path,
            low_pass,
            kernel_radius,
            sigma_color,
            sigma_space,
            diameter,
            edge,
            mode_threshold,
            edge_quotient_threshold,
            show_fig,
            verbose):
    
    with mrcfile.open(tg_path) as mrc:
        data = mrc.data
    pixel_size = read_pixel_size(tg_path)
    data = cp.array(data)

    if data.ndim == 3:
        average = average_along_z(tg_path)
    else:
        average = data

    lp_filtered_img = low_pass_filter_gaussian(average, low_pass, pixel_size)

    lp_bi_img = bilateral_filter(lp_filtered_img, kernel_radius, sigma_color, sigma_space)
    lp_filtered_img_edge = edge_detector(lp_bi_img)
    radius = int((diameter / 2) / pixel_size)
    center_x, center_y, hough_img = hough_transform_for_radius(lp_filtered_img_edge, radius)
    a, b = center_y-radius, center_x-radius
    
    # generate the mask
    mask = flag_masking(lp_filtered_img, a, b, radius, edge)

    mode_diff = find_highest_density(lp_filtered_img, mask.get())

    if show_fig:
        show_figure(hough_img, lp_filtered_img_edge, lp_filtered_img, mask, center_x, center_y, radius)
    
    if verbose:
        print(f"Carbon film edge arc: center_x {center_x}, center_y {center_y}")

    if mode_diff < mode_threshold:
        if verbose:
            print(f"mode_diff: {mode_diff} < {mode_threshold} \n Carbon film not detected!")
        return False
    else:
        # calculate edge_quotient
        edge_map_mean = np.mean(lp_filtered_img_edge)
        masked_edge_map_mean = np.mean(lp_filtered_img_edge[mask.get() == 3])
        if verbose:
            print(f"lp_filtered_img_edge mean: {edge_map_mean}")
            print(f"masked lp_filtered_img_edge mean: {masked_edge_map_mean}")
        edge_quotient = masked_edge_map_mean / edge_map_mean

        if edge_quotient > edge_quotient_threshold:
            if verbose:
                print(f"edge_quotient: {edge_quotient} > {edge_quotient_threshold} \n Carbon film detected!")
            return mask
        else:
            if verbose:
                print(f"edge_quotient: {edge_quotient} < {edge_quotient_threshold} \n Carbon film not detected!")
            return False

        

def show_figure(hough_img, lp_filtered_img_edge, lp_filtered_img, mask, center_x, center_y, radius):
    # plt.figure(1)
    # plt.hist(lp_filtered_img[mask.get() == 1], bins=100, alpha=0.5, label='Inside')
    # plt.hist(lp_filtered_img[mask.get() == 0], bins=100, alpha=0.5, label='Outside')
    # plt.xlabel('Density')
    # plt.ylabel('Frequency')
    # plt.legend()

    a,b = center_y-radius, center_x-radius
    theta = np.linspace(0, 2 * np.pi, num=360, dtype=np.float32)
    x = np.round(a + radius * np.cos(theta))
    y = np.round(b + radius * np.sin(theta))
    valid_indices = (x >= 0) & (x < lp_filtered_img.shape[0]) & (y >= 0) & (y < lp_filtered_img.shape[1])


    # plt.figure(2)
    plt.subplot(141)
    plt.imshow(hough_img, cmap='gray')
    plt.plot(center_x, center_y, 'ro')
    plt.title('Hough Space')

    plt.subplot(142)
    plt.imshow(lp_filtered_img_edge, cmap='gray')
    plt.plot(y[valid_indices], x[valid_indices], 'r-')
    plt.title('Edge Map')
    plt.subplot(143)
    plt.imshow(lp_filtered_img, cmap='gray')
    plt.plot(y[valid_indices], x[valid_indices], 'r-')
    plt.title('Detection Result')

    plt.subplot(144)
    plt.imshow(mask.get(), cmap='gray')
    plt.title('Mask')
    
    plt.show()
    plt.tight_layout()

if __name__ == '__main__':
    detector_for_mrc()